#ifndef MGLPP_EVENT_HPP
#define MGLPP_EVENT_HPP

#include "Keyboard.hpp"
#include "Mouse.hpp"
#include <stdint.h>

/* These have to match the same structures in mgl event */

namespace mgl {
    class Event {
    public:
        struct SizeEvent {
            int width;
            int height;
        };

        struct TextEvent {
            uint32_t codepoint;
            int size;
            char str[5]; /* utf8, null terminated */
        };

        struct KeyEvent {
            Keyboard::Key code;
            bool alt;
            bool control;
            bool shift;
            bool system;
        };

        struct MouseButtonEvent {
            Mouse::Button button;
            int x; /* mouse position relative to left of the window */
            int y; /* mouse position relative to top of the window */
        };

        struct MouseWheelScrollEvent {
            int delta; /* positive = up, negative = down */
            int x; /* mouse position relative to left of the window */
            int y; /* mouse position relative to left of the window */
        };

        struct MouseMoveEvent {
            int x; /* relative to left of the window */
            int y; /* relative to the top of the window */
        };

        enum Type : int {
            Unknown,
            Closed, /* Window closed */
            Resized, /* Window resized */
            LostFocus,
            GainedFocus,
            TextEntered,
            KeyPressed,
            KeyReleased,
            MouseButtonPressed,
            MouseButtonReleased,
            MouseWheelScrolled,
            MouseMoved
        };

        Type type;

        union {
            SizeEvent size;
            TextEvent text;
            KeyEvent key;
            MouseButtonEvent mouse_button;
            MouseWheelScrollEvent mouse_wheel_scroll;
            MouseMoveEvent mouse_move;
        };
    };
}

#endif /* MGLPP_EVENT_HPP */
